import _ from 'lodash/core'

var count = 0

/**
 * Data
 * @class
 */
class Data {
	/**
	 * Data constructor
	 * @constructor
	 * @param {Object} data data, same as "data" in vue options
	 * @param {Object=} options option object
	 * @param {String=} options.name name for Storage name, if not provide will auto generate a name by vuejs-storage count(unstable)
	 * @param {Storage=} options.storage localStorage/sessionStorage or something has similar api, default is window.localStorage
	 * @param {Function=} options.parse json parsing function, default is JSON.parse
	 * @param {Function=} options.stringify json stringifying function, default is JSON.stringify
	 */
	constructor(data, options) {
		this.option = _.defaults(options, {
			parse: JSON.parse,
			stringify: JSON.stringify,
			storage: window.localStorage,
			name: `vuejs-storage-${count++}`
		})
		this.data = data
	}
}
let vuejsStorage = {
	install(Vue, config) {
		Vue.mixin({
			beforeCreate() {
				if ('storage' in this.$options) {
					let { data, option } = this.$options.storage
					let tmp
					if ((tmp = option.storage.getItem(option.name)) !== null) {
						data = _.extend(data, option.parse(tmp))
					}

					this.$options.data = _.extend(this.$options.data, data)

					if (!('watch' in this.$options)) {
						this.$options.watch = {}
					}
					_.each(data, (v_dummy, k) => {
						this.$options.watch[k] = v => {
							data[k] = v
							option.storage.setItem(option.name, option.stringify(data))
						}
					})

				}
			}
		})
	},
	Data
}

export default vuejsStorage